/**********************************************************************
 **                                                                  **
 **                      cnp_server_manager.cpp                      **
 **                                                                  **
 **  Written by:  Alan Richard Wagner                                **
 **                                                                  **
 **  Copyright 2005 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: cnp_server_manager.cpp,v 1.1.1.1 2006/07/12 13:37:55 endo Exp $ */

#include "cnp_server_manager.h"
#include "AbstractCNPTask.h"
#include "CNPTrackTask.h"
#include "CNPInterceptTask.h"
#include "CNPInspectTask.h"
#include "CNPObserveTask.h"
#include "CNPEODTask.h"
#include "CNPRobotConstraint.h" // ENDO - gcc 3.4

extern void CNPPrintf(const char *format, ...);

CNP_Server_Manager::CNP_Server_Manager(void) {}

CNP_OutputBundle CNP_Server_Manager::PremissionCNPBidding(CNP_InputBundle input) {

    CNP_OutputBundle mission; 
    list<AbstractCNPTask*> currentTasks;
    const bool DEBUG = true;

    list<CNP_TaskInfo>::iterator          taskInfoIter; 
    list<AbstractCNPTask*>::iterator      taskIter;

    mission.status = CNP_OUTPUT_NORMAL;

    //Gets a task pointer for each task on the list...
    for(taskInfoIter=input.ListOfCNPTasks.begin();taskInfoIter!=input.ListOfCNPTasks.end();taskInfoIter++) {
        AbstractCNPTask* newTask = NULL; 
        if(strcmp((*taskInfoIter).name.c_str(),"TrackTask") == 0) {
            newTask = new CNPTrackTask();
        }
        else if(strcmp((*taskInfoIter).name.c_str(),"InterceptTask") == 0) {
            newTask = new CNPInterceptTask();
        }	
        else if(strcmp((*taskInfoIter).name.c_str(),"InspectTask") == 0) {
            newTask = new CNPInspectTask();
        }
        else if(strcmp((*taskInfoIter).name.c_str(),"ObserveTask") == 0) {
            newTask = new CNPObserveTask();
        }
        else if(strcmp((*taskInfoIter).name.c_str(),"EODTask") == 0) {
            newTask = new CNPEODTask();
        }
        else {
            CNPPrintf( "Could not locate task type!" );
            continue;
        }

        newTask->SetTaskID((*taskInfoIter).id);
        //check to see if this task has any constraints associated with it...
        list<CNP_TaskConstraint>::iterator              taskCnstIter;

        for(taskCnstIter=input.ListOfCNPTaskConstraints.begin();taskCnstIter!=input.ListOfCNPTaskConstraints.end();taskCnstIter++) {
            //check if the task name is the same as the task the constraint is tied to
            if(strcmp(newTask->GetTaskName().c_str(),(*taskCnstIter).taskInfo.name.c_str() ) == 0) {
                newTask->AddTaskConstraint( (*taskCnstIter) );
            }
        }
        currentTasks.push_back( newTask );
    }

    //Build a list of robot constraint objects
    list<CNP_RobotConstraint>::iterator      rbtIter;
    list<CNPRobotConstraint*>   currentRobots;
 
    input.ListOfCNPRobotConstraints.sort();
  
    CNPRobotConstraint* aRobot = NULL;
    for(rbtIter=input.ListOfCNPRobotConstraints.begin();rbtIter!=input.ListOfCNPRobotConstraints.end();rbtIter++) {
	
        if( aRobot == NULL ) {
            aRobot = new CNPRobotConstraint((*rbtIter).iRobotID );
            currentRobots.push_back( aRobot );
        }
        else if( (*rbtIter).iRobotID != aRobot->GetRobotID() ) {  
            aRobot = new CNPRobotConstraint((*rbtIter).iRobotID );
            currentRobots.push_back( aRobot );
        }
        aRobot->AddConstraint((*rbtIter) );
    }
 
    //At this point we have a list of task objects with each task constraint added to the object and
    //we also have a list of robot objects with constraints added to it. 
    //We must now iterate through all of the tasks and robots and collect bids for each task 
    //Assigning the task to the highest bidder
	if(currentTasks.size() < currentRobots.size()) {
	   mission.status = CNP_OUTPUT_EXCESSIVE_ROBOT;
	}
	
    for(taskIter=currentTasks.begin();taskIter!=currentTasks.end();taskIter++) {
	  
        double dLargestBid = 0;
        int iRobotID = -1; 
        list<CNPRobotConstraint*>::iterator      curRbtIter;
        list<CNPRobotConstraint*>::iterator      selectedRbtIter;
        for(curRbtIter=currentRobots.begin();curRbtIter!=currentRobots.end();curRbtIter++) {

            double dCurrentBid = (*curRbtIter)->BidOnTask( (*taskIter) );

            //have each robot bid on a task
            if (DEBUG) {
                CNPPrintf( "TASKNAME: %s\n",(*taskIter)->GetTaskName().c_str());
                CNPPrintf( "BID: %f\n",dCurrentBid);
            }

            if( dCurrentBid > dLargestBid ) {
                iRobotID = (*curRbtIter)->GetRobotID();
                selectedRbtIter = curRbtIter;
            }
        } 
        if(iRobotID != -1) {
            mission.robotTaskMapping.insert( CNPRobotTaskMapping::value_type(iRobotID, (*taskIter)->GetTaskID()) );
            currentRobots.erase( selectedRbtIter ); 
        }
		else {
		    if(mission.status == CNP_OUTPUT_EXCESSIVE_ROBOT)
			  mission.status = CNP_OUTPUT_EXCESSIVE_ROBOT_AND_INCOMPLETE_MAPPING;
			else
			  mission.status = CNP_OUTPUT_INCOMPLETE_MAPPING;
		}
    }
    //COULD ADD GENERATE AND TEST HERE... If we need to..

    return mission;
}

/**********************************************************************
 * $Log: cnp_server_manager.cpp,v $
 * Revision 1.1.1.1  2006/07/12 13:37:55  endo
 * MissionLab 7.0
 *
 * Revision 1.14  2006/05/14 06:34:26  endo
 * gcc-3.4 upgrade.
 *
 * Revision 1.13  2005/08/12 21:49:01  endo
 * More improvements for August demo.
 *
 * Revision 1.12  2005/08/12 16:43:45  alanwags
 * Added error codes
 *
 * Revision 1.11  2005/08/09 19:12:15  endo
 * Things improved for the August demo.
 *
 * Revision 1.10  2005/07/22 20:01:47  alanwags
 * Fixes for cbr-cnp integration
 *
 * Revision 1.9  2005/07/18 21:02:07  alanwags
 * CBR-CNP Integration fixes
 *
 * Revision 1.8  2005/07/14 19:52:37  alanwags
 * Fixed a bug and added code for other tasks
 *
 * Revision 1.7  2005/07/06 18:11:16  alanwags
 * Added code to fix linking problems. Also added code for intercept and inspect tasks.
 *
 * Revision 1.6  2005/07/03 00:09:01  alanwags
 * Link error fixed
 *
 * Revision 1.5  2005/06/23 22:52:23  alanwags
 * Changes for bidding
 *
 * Revision 1.4  2005/06/22 20:01:12  endo
 * TaskInfo added.
 *
 **********************************************************************/
